==> 1.1.0.zip: terraform-provider-dynamix/internal/service/cloudbroker/disks/flattens/flatten_data_source_cb_disk_list.go <==
package flattens

import (
	"context"
	"encoding/json"
	"fmt"

	"github.com/hashicorp/terraform-plugin-framework/diag"
	"github.com/hashicorp/terraform-plugin-framework/types"
	"github.com/hashicorp/terraform-plugin-log/tflog"
	decort "repository.basistech.ru/BASIS/decort-golang-sdk"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/disks"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/disks/models"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/disks/utilities"
)

// DiskListDataSource flattens data source for disk list.
// Return error in case data source is not found on the platform.
// Flatten errors are added to tflog.
func DiskListDataSource(ctx context.Context, state *models.DisksModel, c *decort.DecortClient) diag.Diagnostics {
	tflog.Info(ctx, "Start flattens.DiskListDataSource")

	diags := diag.Diagnostics{}

	diskList, diags := utilities.DataSourceDiskListCheckPresence(ctx, state, c)
	if diags.HasError() {
		return diags
	}

	tflog.Info(ctx, "flattens.DiskListDataSource: before flatten")

	*state = models.DisksModel{
		ByID:        state.ByID,
		Name:        state.Name,
		AccountName: state.AccountName,
		DiskMaxSize: state.DiskMaxSize,
		Shared:      state.Shared,
		AccountID:   state.AccountID,
		Type:        state.Type,
		SortBy:      state.SortBy,
		Page:        state.Page,
		Size:        state.Size,
		Timeouts:    state.Timeouts,

		// computed fields
		EntryCount: types.Int64Value(int64(diskList.EntryCount)),
	}

	items := make([]models.DiskItemModel, 0, diskList.EntryCount)
	for _, recordDisk := range diskList.Data {
		diskAcl, _ := json.Marshal(recordDisk.ACL)
		d := models.DiskItemModel{
			MachineID:           types.Int64Value(int64(recordDisk.MachineID)),
			MachineName:         types.StringValue(recordDisk.MachineName),
			AccountID:           types.Int64Value(int64(recordDisk.AccountID)),
			AccountName:         types.StringValue(recordDisk.AccountName),
			SEPType:             types.StringValue(recordDisk.SEPType),
			ACL:                 types.StringValue(string(diskAcl)),
			BootPartition:       types.Int64Value(int64(recordDisk.BootPartition)),
			Computes:            flattenComputeList(ctx, recordDisk.Computes),
			CreatedTime:         types.Int64Value(int64(recordDisk.CreatedTime)),
			DeletedTime:         types.Int64Value(int64(recordDisk.DeletedTime)),
			Description:         types.StringValue(recordDisk.Description),
			DestructionTime:     types.Int64Value(int64(recordDisk.DestructionTime)),
			DiskPath:            types.StringValue(recordDisk.DiskPath),
			DeviceName:          types.StringValue(recordDisk.DeviceName),
			GID:                 types.Int64Value(int64(recordDisk.GID)),
			GUID:                types.Int64Value(int64(recordDisk.GUID)),
			ID:                  types.Int64Value(int64(recordDisk.ID)),
			ImageID:             types.Int64Value(int64(recordDisk.ImageID)),
			IQN:                 types.StringValue(recordDisk.IQN),
			Login:               types.StringValue(recordDisk.Login),
			Milestones:          types.Int64Value(int64(recordDisk.Milestones)),
			Name:                types.StringValue(recordDisk.Name),
			Order:               types.Int64Value(int64(recordDisk.Order)),
			Params:              types.StringValue(recordDisk.Params),
			ParentID:            types.Int64Value(int64(recordDisk.ParentID)),
			Password:            types.StringValue(recordDisk.Password),
			PCISlot:             types.Int64Value(int64(recordDisk.PCISlot)),
			Pool:                types.StringValue(recordDisk.Pool),
			PurgeTime:           types.Int64Value(int64(recordDisk.PurgeTime)),
			PurgeAttempts:       types.Int64Value(int64(recordDisk.PurgeAttempts)),
			RealityDeviceNumber: types.Int64Value(int64(recordDisk.RealityDeviceNumber)),
			ReferenceID:         types.StringValue(recordDisk.ReferenceID),
			ResID:               types.StringValue(recordDisk.ResID),
			ResName:             types.StringValue(recordDisk.ResName),
			Role:                types.StringValue(recordDisk.Role),
			SEPID:               types.Int64Value(int64(recordDisk.SEPID)),
			Shareable:           types.BoolValue(recordDisk.Shareable),
			SizeMax:             types.Int64Value(int64(recordDisk.SizeMax)),
			SizeUsed:            types.Float64Value(recordDisk.SizeUsed),
			Snapshots:           flattenSnapshotList(ctx, recordDisk.Snapshots),
			Status:              types.StringValue(recordDisk.Status),
			TechStatus:          types.StringValue(recordDisk.TechStatus),
			Type:                types.StringValue(recordDisk.Type),
			VMID:                types.Int64Value(int64(recordDisk.VMID)),
		}

		d.Images, diags = types.ListValueFrom(ctx, types.StringType, recordDisk.Images)
		if diags != nil {
			tflog.Error(ctx, fmt.Sprint("flattens.DiskListDataSource: cannot flatten recordDisk.Images to d.Images", diags))
		}
		d.PresentTo, diags = types.ListValueFrom(ctx, types.Int64Type, recordDisk.PresentTo)
		if diags != nil {
			tflog.Error(ctx, fmt.Sprint("flattens.DiskListDataSource: cannot flatten recordDisk.PresentTo to d.PresentTo", diags))
		}

		iotune := models.IOModel{
			ReadBytesSec:     types.Int64Value(int64(recordDisk.IOTune.ReadBytesSec)),
			ReadBytesSecMax:  types.Int64Value(int64(recordDisk.IOTune.ReadBytesSecMax)),
			ReadIOPSSec:      types.Int64Value(int64(recordDisk.IOTune.ReadIOPSSec)),
			ReadIOPSSecMax:   types.Int64Value(int64(recordDisk.IOTune.ReadIOPSSecMax)),
			SizeIOPSSec:      types.Int64Value(int64(recordDisk.IOTune.SizeIOPSSec)),
			TotalBytesSec:    types.Int64Value(int64(recordDisk.IOTune.TotalBytesSec)),
			TotalBytesSecMax: types.Int64Value(int64(recordDisk.IOTune.TotalBytesSecMax)),
			TotalIOPSSec:     types.Int64Value(int64(recordDisk.IOTune.TotalIOPSSec)),
			TotalIOPSSecMax:  types.Int64Value(int64(recordDisk.IOTune.TotalIOPSSecMax)),
			WriteBytesSec:    types.Int64Value(int64(recordDisk.IOTune.WriteBytesSec)),
			WriteBytesSecMax: types.Int64Value(int64(recordDisk.IOTune.WriteBytesSecMax)),
			WriteIOPSSec:     types.Int64Value(int64(recordDisk.IOTune.WriteIOPSSec)),
			WriteIOPSSecMax:  types.Int64Value(int64(recordDisk.IOTune.WriteIOPSSecMax)),
		}

		d.IOTune = &iotune

		repl := models.ItemReplModel{
			DiskID:       types.Int64Value(int64(recordDisk.Replication.DiskID)),
			PoolID:       types.StringValue(recordDisk.Replication.PoolID),
			Role:         types.StringValue(recordDisk.Replication.Role),
			SelfVolumeID: types.StringValue(recordDisk.Replication.SelfVolumeID),
			StorageID:    types.StringValue(recordDisk.Replication.StorageID),
			VolumeID:     types.StringValue(recordDisk.Replication.VolumeID),
		}

		d.Replication = &repl

		items = append(items, d)
	}
	state.Items = items

	tflog.Info(ctx, "flattens.DiskListDataSource: end flatten")
	return nil
}

func flattenComputeList(ctx context.Context, computes map[string]string) []models.ComputeModel {
	tflog.Info(ctx, "Start flattenComputeList")

	res := make([]models.ComputeModel, 0, len(computes))

	for k, v := range computes {
		temp := models.ComputeModel{
			ComputeID:   types.StringValue(k),
			ComputeName: types.StringValue(v),
		}

		res = append(res, temp)
	}

	tflog.Info(ctx, "End flattenComputeList")
	return res
}

func flattenSnapshotList(ctx context.Context, snapshots disks.ListSnapshots) []models.SnapshotModel {
	tflog.Info(ctx, "Start flattenSnapshotList")
	res := make([]models.SnapshotModel, 0, len(snapshots))
	for _, item := range snapshots {
		temp := models.SnapshotModel{
			GUID:        types.StringValue(item.GUID),
			Label:       types.StringValue(item.Label),
			ResID:       types.StringValue(item.ResID),
			SnapSetGUID: types.StringValue(item.SnapSetGUID),
			SnapSetTime: types.Int64Value(int64(item.SnapSetTime)),
			Timestamp:   types.Int64Value(int64(item.Timestamp)),
		}
		res = append(res, temp)
	}

	tflog.Info(ctx, "End flattenSnapshotList")
	return res
}

==> 1.2.1.zip: terraform-provider-dynamix/internal/service/cloudbroker/disks/flattens/flatten_data_source_cb_disk_list.go <==
package flattens

import (
	"context"
	"encoding/json"
	"fmt"

	"github.com/hashicorp/terraform-plugin-framework/diag"
	"github.com/hashicorp/terraform-plugin-framework/types"
	"github.com/hashicorp/terraform-plugin-log/tflog"
	"repository.basistech.ru/BASIS/decort-golang-sdk/pkg/cloudbroker/disks"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/client"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/disks/models"
	"repository.basistech.ru/BASIS/terraform-provider-dynamix/internal/service/cloudbroker/disks/utilities"
)

// DiskListDataSource flattens data source for disk list.
// Return error in case data source is not found on the platform.
// Flatten errors are added to tflog.
func DiskListDataSource(ctx context.Context, state *models.DisksModel, c *client.Client) diag.Diagnostics {
	tflog.Info(ctx, "Start flattens.DiskListDataSource")

	diags := diag.Diagnostics{}

	diskList, diags := utilities.DataSourceDiskListCheckPresence(ctx, state, c)
	if diags.HasError() {
		return diags
	}

	tflog.Info(ctx, "flattens.DiskListDataSource: before flatten")

	*state = models.DisksModel{
		ByID:        state.ByID,
		Name:        state.Name,
		AccountName: state.AccountName,
		DiskMaxSize: state.DiskMaxSize,
		Shared:      state.Shared,
		AccountID:   state.AccountID,
		Type:        state.Type,
		SortBy:      state.SortBy,
		Page:        state.Page,
		Size:        state.Size,
		Timeouts:    state.Timeouts,

		// computed fields
		EntryCount: types.Int64Value(int64(diskList.EntryCount)),
	}

	items := make([]models.DiskItemModel, 0, diskList.EntryCount)
	for _, recordDisk := range diskList.Data {
		diskAcl, _ := json.Marshal(recordDisk.ACL)
		d := models.DiskItemModel{
			MachineID:           types.Int64Value(int64(recordDisk.MachineID)),
			MachineName:         types.StringValue(recordDisk.MachineName),
			AccountID:           types.Int64Value(int64(recordDisk.AccountID)),
			AccountName:         types.StringValue(recordDisk.AccountName),
			SEPType:             types.StringValue(recordDisk.SEPType),
			ACL:                 types.StringValue(string(diskAcl)),
			BootPartition:       types.Int64Value(int64(recordDisk.BootPartition)),
			Computes:            flattenComputeList(ctx, recordDisk.Computes),
			CreatedTime:         types.Int64Value(int64(recordDisk.CreatedTime)),
			DeletedTime:         types.Int64Value(int64(recordDisk.DeletedTime)),
			Description:         types.StringValue(recordDisk.Description),
			DestructionTime:     types.Int64Value(int64(recordDisk.DestructionTime)),
			DiskPath:            types.StringValue(recordDisk.DiskPath),
			DeviceName:          types.StringValue(recordDisk.DeviceName),
			GID:                 types.Int64Value(int64(recordDisk.GID)),
			GUID:                types.Int64Value(int64(recordDisk.GUID)),
			ID:                  types.Int64Value(int64(recordDisk.ID)),
			ImageID:             types.Int64Value(int64(recordDisk.ImageID)),
			IQN:                 types.StringValue(recordDisk.IQN),
			Login:               types.StringValue(recordDisk.Login),
			Milestones:          types.Int64Value(int64(recordDisk.Milestones)),
			Name:                types.StringValue(recordDisk.Name),
			Order:               types.Int64Value(int64(recordDisk.Order)),
			Params:              types.StringValue(recordDisk.Params),
			ParentID:            types.Int64Value(int64(recordDisk.ParentID)),
			Password:            types.StringValue(recordDisk.Password),
			PCISlot:             types.Int64Value(int64(recordDisk.PCISlot)),
			Pool:                types.StringValue(recordDisk.Pool),
			PurgeTime:           types.Int64Value(int64(recordDisk.PurgeTime)),
			PurgeAttempts:       types.Int64Value(int64(recordDisk.PurgeAttempts)),
			RealityDeviceNumber: types.Int64Value(int64(recordDisk.RealityDeviceNumber)),
			ReferenceID:         types.StringValue(recordDisk.ReferenceID),
			ResID:               types.StringValue(recordDisk.ResID),
			ResName:             types.StringValue(recordDisk.ResName),
			Role:                types.StringValue(recordDisk.Role),
			SEPID:               types.Int64Value(int64(recordDisk.SEPID)),
			Shareable:           types.BoolValue(recordDisk.Shareable),
			SizeMax:             types.Int64Value(int64(recordDisk.SizeMax)),
			SizeUsed:            types.Float64Value(recordDisk.SizeUsed),
			Snapshots:           flattenSnapshotList(ctx, recordDisk.Snapshots),
			Status:              types.StringValue(recordDisk.Status),
			TechStatus:          types.StringValue(recordDisk.TechStatus),
			Type:                types.StringValue(recordDisk.Type),
			VMID:                types.Int64Value(int64(recordDisk.VMID)),
		}

		d.Images, diags = types.ListValueFrom(ctx, types.StringType, recordDisk.Images)
		if diags != nil {
			tflog.Error(ctx, fmt.Sprint("flattens.DiskListDataSource: cannot flatten recordDisk.Images to d.Images", diags))
		}
		d.PresentTo, diags = types.ListValueFrom(ctx, types.Int64Type, recordDisk.PresentTo)
		if diags != nil {
			tflog.Error(ctx, fmt.Sprint("flattens.DiskListDataSource: cannot flatten recordDisk.PresentTo to d.PresentTo", diags))
		}

		iotune := models.IOModel{
			ReadBytesSec:     types.Int64Value(int64(recordDisk.IOTune.ReadBytesSec)),
			ReadBytesSecMax:  types.Int64Value(int64(recordDisk.IOTune.ReadBytesSecMax)),
			ReadIOPSSec:      types.Int64Value(int64(recordDisk.IOTune.ReadIOPSSec)),
			ReadIOPSSecMax:   types.Int64Value(int64(recordDisk.IOTune.ReadIOPSSecMax)),
			SizeIOPSSec:      types.Int64Value(int64(recordDisk.IOTune.SizeIOPSSec)),
			TotalBytesSec:    types.Int64Value(int64(recordDisk.IOTune.TotalBytesSec)),
			TotalBytesSecMax: types.Int64Value(int64(recordDisk.IOTune.TotalBytesSecMax)),
			TotalIOPSSec:     types.Int64Value(int64(recordDisk.IOTune.TotalIOPSSec)),
			TotalIOPSSecMax:  types.Int64Value(int64(recordDisk.IOTune.TotalIOPSSecMax)),
			WriteBytesSec:    types.Int64Value(int64(recordDisk.IOTune.WriteBytesSec)),
			WriteBytesSecMax: types.Int64Value(int64(recordDisk.IOTune.WriteBytesSecMax)),
			WriteIOPSSec:     types.Int64Value(int64(recordDisk.IOTune.WriteIOPSSec)),
			WriteIOPSSecMax:  types.Int64Value(int64(recordDisk.IOTune.WriteIOPSSecMax)),
		}

		d.IOTune = &iotune

		repl := models.ItemReplModel{
			DiskID:       types.Int64Value(int64(recordDisk.Replication.DiskID)),
			PoolID:       types.StringValue(recordDisk.Replication.PoolID),
			Role:         types.StringValue(recordDisk.Replication.Role),
			SelfVolumeID: types.StringValue(recordDisk.Replication.SelfVolumeID),
			StorageID:    types.StringValue(recordDisk.Replication.StorageID),
			VolumeID:     types.StringValue(recordDisk.Replication.VolumeID),
		}

		d.Replication = &repl

		items = append(items, d)
	}
	state.Items = items

	tflog.Info(ctx, "flattens.DiskListDataSource: end flatten")
	return nil
}

func flattenComputeList(ctx context.Context, computes map[string]string) []models.ComputeModel {
	tflog.Info(ctx, "Start flattenComputeList")

	res := make([]models.ComputeModel, 0, len(computes))

	for k, v := range computes {
		temp := models.ComputeModel{
			ComputeID:   types.StringValue(k),
			ComputeName: types.StringValue(v),
		}

		res = append(res, temp)
	}

	tflog.Info(ctx, "End flattenComputeList")
	return res
}

func flattenSnapshotList(ctx context.Context, snapshots disks.ListSnapshots) []models.SnapshotModel {
	tflog.Info(ctx, "Start flattenSnapshotList")
	res := make([]models.SnapshotModel, 0, len(snapshots))
	for _, item := range snapshots {
		temp := models.SnapshotModel{
			GUID:        types.StringValue(item.GUID),
			Label:       types.StringValue(item.Label),
			ResID:       types.StringValue(item.ResID),
			SnapSetGUID: types.StringValue(item.SnapSetGUID),
			SnapSetTime: types.Int64Value(int64(item.SnapSetTime)),
			Timestamp:   types.Int64Value(int64(item.Timestamp)),
		}
		res = append(res, temp)
	}

	tflog.Info(ctx, "End flattenSnapshotList")
	return res
}
